package io.syndesis.server.metrics.prometheus;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.common.base.MoreObjects;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Lists;
import com.google.errorprone.annotations.CanIgnoreReturnValue;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.util.List;
import java.util.Objects;
import javax.annotation.CheckReturnValue;
import javax.annotation.Generated;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;

/**
 * Immutable implementation of {@link QueryResult.Data}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new QueryResult.Data.Builder()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@Generated({"Immutables.generator", "QueryResult.Data"})
@Immutable
@CheckReturnValue
public final class ImmutableData implements QueryResult.Data {
  private final String resultType;
  private final ImmutableList<QueryResult.Data.Result> result;

  private ImmutableData(
      String resultType,
      ImmutableList<QueryResult.Data.Result> result) {
    this.resultType = resultType;
    this.result = result;
  }

  /**
   * @return The value of the {@code resultType} attribute
   */
  @JsonProperty("resultType")
  @Override
  public String getResultType() {
    return resultType;
  }

  /**
   * @return The value of the {@code result} attribute
   */
  @JsonProperty("result")
  @Override
  public ImmutableList<QueryResult.Data.Result> getResult() {
    return result;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link QueryResult.Data#getResultType() resultType} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for resultType
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableData withResultType(String value) {
    if (this.resultType.equals(value)) return this;
    String newValue = Objects.requireNonNull(value, "resultType");
    return new ImmutableData(newValue, this.result);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link QueryResult.Data#getResult() result}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableData withResult(QueryResult.Data.Result... elements) {
    ImmutableList<QueryResult.Data.Result> newValue = ImmutableList.copyOf(elements);
    return new ImmutableData(this.resultType, newValue);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link QueryResult.Data#getResult() result}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of result elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableData withResult(Iterable<? extends QueryResult.Data.Result> elements) {
    if (this.result == elements) return this;
    ImmutableList<QueryResult.Data.Result> newValue = ImmutableList.copyOf(elements);
    return new ImmutableData(this.resultType, newValue);
  }

  /**
   * This instance is equal to all instances of {@code ImmutableData} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableData
        && equalTo((ImmutableData) another);
  }

  private boolean equalTo(ImmutableData another) {
    return resultType.equals(another.resultType)
        && result.equals(another.result);
  }

  /**
   * Computes a hash code from attributes: {@code resultType}, {@code result}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + resultType.hashCode();
    h += (h << 5) + result.hashCode();
    return h;
  }

  /**
   * Prints the immutable value {@code Data} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return MoreObjects.toStringHelper("Data")
        .omitNullValues()
        .add("resultType", resultType)
        .add("result", result)
        .toString();
  }

  /**
   * Creates an immutable copy of a {@link QueryResult.Data} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable Data instance
   */
  public static ImmutableData copyOf(QueryResult.Data instance) {
    if (instance instanceof ImmutableData) {
      return (ImmutableData) instance;
    }
    return new QueryResult.Data.Builder()
        .from(instance)
        .build();
  }

  /**
   * Builds instances of type {@link ImmutableData ImmutableData}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @NotThreadSafe
  public static class Builder {
    private static final long INIT_BIT_RESULT_TYPE = 0x1L;
    private long initBits = 0x1L;

    private @Nullable String resultType;
    private ImmutableList.Builder<QueryResult.Data.Result> result = ImmutableList.builder();

    /**
     * Creates a builder for {@link ImmutableData ImmutableData} instances.
     */
    public Builder() {
      if (!(this instanceof QueryResult.Data.Builder)) {
        throw new UnsupportedOperationException("Use: new QueryResult.Data.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code Data} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * Collection elements and entries will be added, not replaced.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final QueryResult.Data.Builder from(QueryResult.Data instance) {
      Objects.requireNonNull(instance, "instance");
      resultType(instance.getResultType());
      addAllResult(instance.getResult());
      return (QueryResult.Data.Builder) this;
    }

    /**
     * Initializes the value for the {@link QueryResult.Data#getResultType() resultType} attribute.
     * @param resultType The value for resultType 
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("resultType")
    public final QueryResult.Data.Builder resultType(String resultType) {
      this.resultType = Objects.requireNonNull(resultType, "resultType");
      initBits &= ~INIT_BIT_RESULT_TYPE;
      return (QueryResult.Data.Builder) this;
    }

    /**
     * Adds one element to {@link QueryResult.Data#getResult() result} list.
     * @param element A result element
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final QueryResult.Data.Builder addResult(QueryResult.Data.Result element) {
      this.result.add(element);
      return (QueryResult.Data.Builder) this;
    }

    /**
     * Adds elements to {@link QueryResult.Data#getResult() result} list.
     * @param elements An array of result elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final QueryResult.Data.Builder addResult(QueryResult.Data.Result... elements) {
      this.result.add(elements);
      return (QueryResult.Data.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link QueryResult.Data#getResult() result} list.
     * @param elements An iterable of result elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("result")
    public final QueryResult.Data.Builder result(Iterable<? extends QueryResult.Data.Result> elements) {
      this.result = ImmutableList.builder();
      return addAllResult(elements);
    }

    /**
     * Adds elements to {@link QueryResult.Data#getResult() result} list.
     * @param elements An iterable of result elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final QueryResult.Data.Builder addAllResult(Iterable<? extends QueryResult.Data.Result> elements) {
      this.result.addAll(elements);
      return (QueryResult.Data.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableData ImmutableData}.
     * @return An immutable instance of Data
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableData build() {
      if (initBits != 0) {
        throw new IllegalStateException(formatRequiredAttributesMessage());
      }
      return new ImmutableData(resultType, result.build());
    }

    private String formatRequiredAttributesMessage() {
      List<String> attributes = Lists.newArrayList();
      if ((initBits & INIT_BIT_RESULT_TYPE) != 0) attributes.add("resultType");
      return "Cannot build Data, some of required attributes are not set " + attributes;
    }
  }
}
