package io.syndesis.server.metrics.prometheus;

import com.google.common.base.MoreObjects;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Lists;
import com.google.errorprone.annotations.CanIgnoreReturnValue;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import javax.annotation.CheckReturnValue;
import javax.annotation.Generated;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;

/**
 * Immutable implementation of {@link HttpQuery}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new HttpQuery.Builder()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@Generated({"Immutables.generator", "HttpQuery"})
@Immutable
@CheckReturnValue
public final class ImmutableHttpQuery implements HttpQuery {
  private final String host;
  private final @Nullable String function;
  private final @Nullable String aggregationOperator;
  private final String metric;
  private final ImmutableList<HttpQuery.LabelValue> labelValues;
  private final @Nullable String range;
  private final ImmutableList<String> withoutLabels;
  private final ImmutableList<String> byLabels;

  private ImmutableHttpQuery(
      String host,
      @Nullable String function,
      @Nullable String aggregationOperator,
      String metric,
      ImmutableList<HttpQuery.LabelValue> labelValues,
      @Nullable String range,
      ImmutableList<String> withoutLabels,
      ImmutableList<String> byLabels) {
    this.host = host;
    this.function = function;
    this.aggregationOperator = aggregationOperator;
    this.metric = metric;
    this.labelValues = labelValues;
    this.range = range;
    this.withoutLabels = withoutLabels;
    this.byLabels = byLabels;
  }

  /**
   * @return The value of the {@code host} attribute
   */
  @Override
  public String getHost() {
    return host;
  }

  /**
   * @return The value of the {@code function} attribute
   */
  @Override
  public Optional<String> getFunction() {
    return Optional.ofNullable(function);
  }

  /**
   * @return The value of the {@code aggregationOperator} attribute
   */
  @Override
  public Optional<String> getAggregationOperator() {
    return Optional.ofNullable(aggregationOperator);
  }

  /**
   * @return The value of the {@code metric} attribute
   */
  @Override
  public String getMetric() {
    return metric;
  }

  /**
   * @return The value of the {@code labelValues} attribute
   */
  @Override
  public ImmutableList<HttpQuery.LabelValue> getLabelValues() {
    return labelValues;
  }

  /**
   * @return The value of the {@code range} attribute
   */
  @Override
  public Optional<String> getRange() {
    return Optional.ofNullable(range);
  }

  /**
   * @return The value of the {@code withoutLabels} attribute
   */
  @Override
  public ImmutableList<String> getWithoutLabels() {
    return withoutLabels;
  }

  /**
   * @return The value of the {@code byLabels} attribute
   */
  @Override
  public ImmutableList<String> getByLabels() {
    return byLabels;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link HttpQuery#getHost() host} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for host
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableHttpQuery withHost(String value) {
    if (this.host.equals(value)) return this;
    String newValue = Objects.requireNonNull(value, "host");
    return new ImmutableHttpQuery(
        newValue,
        this.function,
        this.aggregationOperator,
        this.metric,
        this.labelValues,
        this.range,
        this.withoutLabels,
        this.byLabels);
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link HttpQuery#getFunction() function} attribute.
   * @param value The value for function
   * @return A modified copy of {@code this} object
   */
  public final ImmutableHttpQuery withFunction(String value) {
    @Nullable String newValue = Objects.requireNonNull(value, "function");
    if (Objects.equals(this.function, newValue)) return this;
    return new ImmutableHttpQuery(
        this.host,
        newValue,
        this.aggregationOperator,
        this.metric,
        this.labelValues,
        this.range,
        this.withoutLabels,
        this.byLabels);
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link HttpQuery#getFunction() function} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for function
   * @return A modified copy of {@code this} object
   */
  public final ImmutableHttpQuery withFunction(Optional<String> optional) {
    @Nullable String value = optional.orElse(null);
    if (Objects.equals(this.function, value)) return this;
    return new ImmutableHttpQuery(
        this.host,
        value,
        this.aggregationOperator,
        this.metric,
        this.labelValues,
        this.range,
        this.withoutLabels,
        this.byLabels);
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link HttpQuery#getAggregationOperator() aggregationOperator} attribute.
   * @param value The value for aggregationOperator
   * @return A modified copy of {@code this} object
   */
  public final ImmutableHttpQuery withAggregationOperator(String value) {
    @Nullable String newValue = Objects.requireNonNull(value, "aggregationOperator");
    if (Objects.equals(this.aggregationOperator, newValue)) return this;
    return new ImmutableHttpQuery(
        this.host,
        this.function,
        newValue,
        this.metric,
        this.labelValues,
        this.range,
        this.withoutLabels,
        this.byLabels);
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link HttpQuery#getAggregationOperator() aggregationOperator} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for aggregationOperator
   * @return A modified copy of {@code this} object
   */
  public final ImmutableHttpQuery withAggregationOperator(Optional<String> optional) {
    @Nullable String value = optional.orElse(null);
    if (Objects.equals(this.aggregationOperator, value)) return this;
    return new ImmutableHttpQuery(
        this.host,
        this.function,
        value,
        this.metric,
        this.labelValues,
        this.range,
        this.withoutLabels,
        this.byLabels);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link HttpQuery#getMetric() metric} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for metric
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableHttpQuery withMetric(String value) {
    if (this.metric.equals(value)) return this;
    String newValue = Objects.requireNonNull(value, "metric");
    return new ImmutableHttpQuery(
        this.host,
        this.function,
        this.aggregationOperator,
        newValue,
        this.labelValues,
        this.range,
        this.withoutLabels,
        this.byLabels);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link HttpQuery#getLabelValues() labelValues}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableHttpQuery withLabelValues(HttpQuery.LabelValue... elements) {
    ImmutableList<HttpQuery.LabelValue> newValue = ImmutableList.copyOf(elements);
    return new ImmutableHttpQuery(
        this.host,
        this.function,
        this.aggregationOperator,
        this.metric,
        newValue,
        this.range,
        this.withoutLabels,
        this.byLabels);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link HttpQuery#getLabelValues() labelValues}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of labelValues elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableHttpQuery withLabelValues(Iterable<? extends HttpQuery.LabelValue> elements) {
    if (this.labelValues == elements) return this;
    ImmutableList<HttpQuery.LabelValue> newValue = ImmutableList.copyOf(elements);
    return new ImmutableHttpQuery(
        this.host,
        this.function,
        this.aggregationOperator,
        this.metric,
        newValue,
        this.range,
        this.withoutLabels,
        this.byLabels);
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link HttpQuery#getRange() range} attribute.
   * @param value The value for range
   * @return A modified copy of {@code this} object
   */
  public final ImmutableHttpQuery withRange(String value) {
    @Nullable String newValue = Objects.requireNonNull(value, "range");
    if (Objects.equals(this.range, newValue)) return this;
    return new ImmutableHttpQuery(
        this.host,
        this.function,
        this.aggregationOperator,
        this.metric,
        this.labelValues,
        newValue,
        this.withoutLabels,
        this.byLabels);
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link HttpQuery#getRange() range} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for range
   * @return A modified copy of {@code this} object
   */
  public final ImmutableHttpQuery withRange(Optional<String> optional) {
    @Nullable String value = optional.orElse(null);
    if (Objects.equals(this.range, value)) return this;
    return new ImmutableHttpQuery(
        this.host,
        this.function,
        this.aggregationOperator,
        this.metric,
        this.labelValues,
        value,
        this.withoutLabels,
        this.byLabels);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link HttpQuery#getWithoutLabels() withoutLabels}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableHttpQuery withWithoutLabels(String... elements) {
    ImmutableList<String> newValue = ImmutableList.copyOf(elements);
    return new ImmutableHttpQuery(
        this.host,
        this.function,
        this.aggregationOperator,
        this.metric,
        this.labelValues,
        this.range,
        newValue,
        this.byLabels);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link HttpQuery#getWithoutLabels() withoutLabels}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of withoutLabels elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableHttpQuery withWithoutLabels(Iterable<String> elements) {
    if (this.withoutLabels == elements) return this;
    ImmutableList<String> newValue = ImmutableList.copyOf(elements);
    return new ImmutableHttpQuery(
        this.host,
        this.function,
        this.aggregationOperator,
        this.metric,
        this.labelValues,
        this.range,
        newValue,
        this.byLabels);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link HttpQuery#getByLabels() byLabels}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableHttpQuery withByLabels(String... elements) {
    ImmutableList<String> newValue = ImmutableList.copyOf(elements);
    return new ImmutableHttpQuery(
        this.host,
        this.function,
        this.aggregationOperator,
        this.metric,
        this.labelValues,
        this.range,
        this.withoutLabels,
        newValue);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link HttpQuery#getByLabels() byLabels}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of byLabels elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableHttpQuery withByLabels(Iterable<String> elements) {
    if (this.byLabels == elements) return this;
    ImmutableList<String> newValue = ImmutableList.copyOf(elements);
    return new ImmutableHttpQuery(
        this.host,
        this.function,
        this.aggregationOperator,
        this.metric,
        this.labelValues,
        this.range,
        this.withoutLabels,
        newValue);
  }

  /**
   * This instance is equal to all instances of {@code ImmutableHttpQuery} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableHttpQuery
        && equalTo((ImmutableHttpQuery) another);
  }

  private boolean equalTo(ImmutableHttpQuery another) {
    return host.equals(another.host)
        && Objects.equals(function, another.function)
        && Objects.equals(aggregationOperator, another.aggregationOperator)
        && metric.equals(another.metric)
        && labelValues.equals(another.labelValues)
        && Objects.equals(range, another.range)
        && withoutLabels.equals(another.withoutLabels)
        && byLabels.equals(another.byLabels);
  }

  /**
   * Computes a hash code from attributes: {@code host}, {@code function}, {@code aggregationOperator}, {@code metric}, {@code labelValues}, {@code range}, {@code withoutLabels}, {@code byLabels}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + host.hashCode();
    h += (h << 5) + Objects.hashCode(function);
    h += (h << 5) + Objects.hashCode(aggregationOperator);
    h += (h << 5) + metric.hashCode();
    h += (h << 5) + labelValues.hashCode();
    h += (h << 5) + Objects.hashCode(range);
    h += (h << 5) + withoutLabels.hashCode();
    h += (h << 5) + byLabels.hashCode();
    return h;
  }

  /**
   * Prints the immutable value {@code HttpQuery} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return MoreObjects.toStringHelper("HttpQuery")
        .omitNullValues()
        .add("host", host)
        .add("function", function)
        .add("aggregationOperator", aggregationOperator)
        .add("metric", metric)
        .add("labelValues", labelValues)
        .add("range", range)
        .add("withoutLabels", withoutLabels)
        .add("byLabels", byLabels)
        .toString();
  }

  /**
   * Creates an immutable copy of a {@link HttpQuery} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable HttpQuery instance
   */
  public static ImmutableHttpQuery copyOf(HttpQuery instance) {
    if (instance instanceof ImmutableHttpQuery) {
      return (ImmutableHttpQuery) instance;
    }
    return new HttpQuery.Builder()
        .from(instance)
        .build();
  }

  /**
   * Builds instances of type {@link ImmutableHttpQuery ImmutableHttpQuery}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @NotThreadSafe
  public static class Builder {
    private static final long INIT_BIT_HOST = 0x1L;
    private static final long INIT_BIT_METRIC = 0x2L;
    private long initBits = 0x3L;

    private @Nullable String host;
    private @Nullable String function;
    private @Nullable String aggregationOperator;
    private @Nullable String metric;
    private ImmutableList.Builder<HttpQuery.LabelValue> labelValues = ImmutableList.builder();
    private @Nullable String range;
    private ImmutableList.Builder<String> withoutLabels = ImmutableList.builder();
    private ImmutableList.Builder<String> byLabels = ImmutableList.builder();

    /**
     * Creates a builder for {@link ImmutableHttpQuery ImmutableHttpQuery} instances.
     */
    public Builder() {
      if (!(this instanceof HttpQuery.Builder)) {
        throw new UnsupportedOperationException("Use: new HttpQuery.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code HttpQuery} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * Collection elements and entries will be added, not replaced.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.Builder from(HttpQuery instance) {
      Objects.requireNonNull(instance, "instance");
      host(instance.getHost());
      Optional<String> functionOptional = instance.getFunction();
      if (functionOptional.isPresent()) {
        function(functionOptional);
      }
      Optional<String> aggregationOperatorOptional = instance.getAggregationOperator();
      if (aggregationOperatorOptional.isPresent()) {
        aggregationOperator(aggregationOperatorOptional);
      }
      metric(instance.getMetric());
      addAllLabelValues(instance.getLabelValues());
      Optional<String> rangeOptional = instance.getRange();
      if (rangeOptional.isPresent()) {
        range(rangeOptional);
      }
      addAllWithoutLabels(instance.getWithoutLabels());
      addAllByLabels(instance.getByLabels());
      return (HttpQuery.Builder) this;
    }

    /**
     * Initializes the value for the {@link HttpQuery#getHost() host} attribute.
     * @param host The value for host 
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.Builder host(String host) {
      this.host = Objects.requireNonNull(host, "host");
      initBits &= ~INIT_BIT_HOST;
      return (HttpQuery.Builder) this;
    }

    /**
     * Initializes the optional value {@link HttpQuery#getFunction() function} to function.
     * @param function The value for function
     * @return {@code this} builder for chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.Builder function(String function) {
      this.function = Objects.requireNonNull(function, "function");
      return (HttpQuery.Builder) this;
    }

    /**
     * Initializes the optional value {@link HttpQuery#getFunction() function} to function.
     * @param function The value for function
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.Builder function(Optional<String> function) {
      this.function = function.orElse(null);
      return (HttpQuery.Builder) this;
    }

    /**
     * Initializes the optional value {@link HttpQuery#getAggregationOperator() aggregationOperator} to aggregationOperator.
     * @param aggregationOperator The value for aggregationOperator
     * @return {@code this} builder for chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.Builder aggregationOperator(String aggregationOperator) {
      this.aggregationOperator = Objects.requireNonNull(aggregationOperator, "aggregationOperator");
      return (HttpQuery.Builder) this;
    }

    /**
     * Initializes the optional value {@link HttpQuery#getAggregationOperator() aggregationOperator} to aggregationOperator.
     * @param aggregationOperator The value for aggregationOperator
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.Builder aggregationOperator(Optional<String> aggregationOperator) {
      this.aggregationOperator = aggregationOperator.orElse(null);
      return (HttpQuery.Builder) this;
    }

    /**
     * Initializes the value for the {@link HttpQuery#getMetric() metric} attribute.
     * @param metric The value for metric 
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.Builder metric(String metric) {
      this.metric = Objects.requireNonNull(metric, "metric");
      initBits &= ~INIT_BIT_METRIC;
      return (HttpQuery.Builder) this;
    }

    /**
     * Adds one element to {@link HttpQuery#getLabelValues() labelValues} list.
     * @param element A labelValues element
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.Builder addLabelValues(HttpQuery.LabelValue element) {
      this.labelValues.add(element);
      return (HttpQuery.Builder) this;
    }

    /**
     * Adds elements to {@link HttpQuery#getLabelValues() labelValues} list.
     * @param elements An array of labelValues elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.Builder addLabelValues(HttpQuery.LabelValue... elements) {
      this.labelValues.add(elements);
      return (HttpQuery.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link HttpQuery#getLabelValues() labelValues} list.
     * @param elements An iterable of labelValues elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.Builder labelValues(Iterable<? extends HttpQuery.LabelValue> elements) {
      this.labelValues = ImmutableList.builder();
      return addAllLabelValues(elements);
    }

    /**
     * Adds elements to {@link HttpQuery#getLabelValues() labelValues} list.
     * @param elements An iterable of labelValues elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.Builder addAllLabelValues(Iterable<? extends HttpQuery.LabelValue> elements) {
      this.labelValues.addAll(elements);
      return (HttpQuery.Builder) this;
    }

    /**
     * Initializes the optional value {@link HttpQuery#getRange() range} to range.
     * @param range The value for range
     * @return {@code this} builder for chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.Builder range(String range) {
      this.range = Objects.requireNonNull(range, "range");
      return (HttpQuery.Builder) this;
    }

    /**
     * Initializes the optional value {@link HttpQuery#getRange() range} to range.
     * @param range The value for range
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.Builder range(Optional<String> range) {
      this.range = range.orElse(null);
      return (HttpQuery.Builder) this;
    }

    /**
     * Adds one element to {@link HttpQuery#getWithoutLabels() withoutLabels} list.
     * @param element A withoutLabels element
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.Builder addWithoutLabels(String element) {
      this.withoutLabels.add(element);
      return (HttpQuery.Builder) this;
    }

    /**
     * Adds elements to {@link HttpQuery#getWithoutLabels() withoutLabels} list.
     * @param elements An array of withoutLabels elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.Builder addWithoutLabels(String... elements) {
      this.withoutLabels.add(elements);
      return (HttpQuery.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link HttpQuery#getWithoutLabels() withoutLabels} list.
     * @param elements An iterable of withoutLabels elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.Builder withoutLabels(Iterable<String> elements) {
      this.withoutLabels = ImmutableList.builder();
      return addAllWithoutLabels(elements);
    }

    /**
     * Adds elements to {@link HttpQuery#getWithoutLabels() withoutLabels} list.
     * @param elements An iterable of withoutLabels elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.Builder addAllWithoutLabels(Iterable<String> elements) {
      this.withoutLabels.addAll(elements);
      return (HttpQuery.Builder) this;
    }

    /**
     * Adds one element to {@link HttpQuery#getByLabels() byLabels} list.
     * @param element A byLabels element
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.Builder addByLabels(String element) {
      this.byLabels.add(element);
      return (HttpQuery.Builder) this;
    }

    /**
     * Adds elements to {@link HttpQuery#getByLabels() byLabels} list.
     * @param elements An array of byLabels elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.Builder addByLabels(String... elements) {
      this.byLabels.add(elements);
      return (HttpQuery.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link HttpQuery#getByLabels() byLabels} list.
     * @param elements An iterable of byLabels elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.Builder byLabels(Iterable<String> elements) {
      this.byLabels = ImmutableList.builder();
      return addAllByLabels(elements);
    }

    /**
     * Adds elements to {@link HttpQuery#getByLabels() byLabels} list.
     * @param elements An iterable of byLabels elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.Builder addAllByLabels(Iterable<String> elements) {
      this.byLabels.addAll(elements);
      return (HttpQuery.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableHttpQuery ImmutableHttpQuery}.
     * @return An immutable instance of HttpQuery
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableHttpQuery build() {
      if (initBits != 0) {
        throw new IllegalStateException(formatRequiredAttributesMessage());
      }
      return new ImmutableHttpQuery(
          host,
          function,
          aggregationOperator,
          metric,
          labelValues.build(),
          range,
          withoutLabels.build(),
          byLabels.build());
    }

    private String formatRequiredAttributesMessage() {
      List<String> attributes = Lists.newArrayList();
      if ((initBits & INIT_BIT_HOST) != 0) attributes.add("host");
      if ((initBits & INIT_BIT_METRIC) != 0) attributes.add("metric");
      return "Cannot build HttpQuery, some of required attributes are not set " + attributes;
    }
  }
}
