package io.syndesis.server.metrics.prometheus;

import com.google.common.base.MoreObjects;
import com.google.common.collect.Lists;
import com.google.errorprone.annotations.CanIgnoreReturnValue;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.util.List;
import java.util.Objects;
import javax.annotation.CheckReturnValue;
import javax.annotation.Generated;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;

/**
 * Immutable implementation of {@link HttpQuery.LabelValue}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new HttpQuery.LabelValue.Builder()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@Generated({"Immutables.generator", "HttpQuery.LabelValue"})
@Immutable
@CheckReturnValue
public final class ImmutableLabelValue implements HttpQuery.LabelValue {
  private final String label;
  private final String value;

  private ImmutableLabelValue(String label, String value) {
    this.label = label;
    this.value = value;
  }

  /**
   * @return The value of the {@code label} attribute
   */
  @Override
  public String getLabel() {
    return label;
  }

  /**
   * @return The value of the {@code value} attribute
   */
  @Override
  public String getValue() {
    return value;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link HttpQuery.LabelValue#getLabel() label} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for label
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableLabelValue withLabel(String value) {
    if (this.label.equals(value)) return this;
    String newValue = Objects.requireNonNull(value, "label");
    return new ImmutableLabelValue(newValue, this.value);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link HttpQuery.LabelValue#getValue() value} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for value
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableLabelValue withValue(String value) {
    if (this.value.equals(value)) return this;
    String newValue = Objects.requireNonNull(value, "value");
    return new ImmutableLabelValue(this.label, newValue);
  }

  /**
   * This instance is equal to all instances of {@code ImmutableLabelValue} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableLabelValue
        && equalTo((ImmutableLabelValue) another);
  }

  private boolean equalTo(ImmutableLabelValue another) {
    return label.equals(another.label)
        && value.equals(another.value);
  }

  /**
   * Computes a hash code from attributes: {@code label}, {@code value}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + label.hashCode();
    h += (h << 5) + value.hashCode();
    return h;
  }

  /**
   * Prints the immutable value {@code LabelValue} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return MoreObjects.toStringHelper("LabelValue")
        .omitNullValues()
        .add("label", label)
        .add("value", value)
        .toString();
  }

  /**
   * Creates an immutable copy of a {@link HttpQuery.LabelValue} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable LabelValue instance
   */
  public static ImmutableLabelValue copyOf(HttpQuery.LabelValue instance) {
    if (instance instanceof ImmutableLabelValue) {
      return (ImmutableLabelValue) instance;
    }
    return new HttpQuery.LabelValue.Builder()
        .from(instance)
        .build();
  }

  /**
   * Builds instances of type {@link ImmutableLabelValue ImmutableLabelValue}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @NotThreadSafe
  public static class Builder {
    private static final long INIT_BIT_LABEL = 0x1L;
    private static final long INIT_BIT_VALUE = 0x2L;
    private long initBits = 0x3L;

    private @Nullable String label;
    private @Nullable String value;

    /**
     * Creates a builder for {@link ImmutableLabelValue ImmutableLabelValue} instances.
     */
    public Builder() {
      if (!(this instanceof HttpQuery.LabelValue.Builder)) {
        throw new UnsupportedOperationException("Use: new HttpQuery.LabelValue.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code LabelValue} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.LabelValue.Builder from(HttpQuery.LabelValue instance) {
      Objects.requireNonNull(instance, "instance");
      label(instance.getLabel());
      value(instance.getValue());
      return (HttpQuery.LabelValue.Builder) this;
    }

    /**
     * Initializes the value for the {@link HttpQuery.LabelValue#getLabel() label} attribute.
     * @param label The value for label 
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.LabelValue.Builder label(String label) {
      this.label = Objects.requireNonNull(label, "label");
      initBits &= ~INIT_BIT_LABEL;
      return (HttpQuery.LabelValue.Builder) this;
    }

    /**
     * Initializes the value for the {@link HttpQuery.LabelValue#getValue() value} attribute.
     * @param value The value for value 
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final HttpQuery.LabelValue.Builder value(String value) {
      this.value = Objects.requireNonNull(value, "value");
      initBits &= ~INIT_BIT_VALUE;
      return (HttpQuery.LabelValue.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableLabelValue ImmutableLabelValue}.
     * @return An immutable instance of LabelValue
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableLabelValue build() {
      if (initBits != 0) {
        throw new IllegalStateException(formatRequiredAttributesMessage());
      }
      return new ImmutableLabelValue(label, value);
    }

    private String formatRequiredAttributesMessage() {
      List<String> attributes = Lists.newArrayList();
      if ((initBits & INIT_BIT_LABEL) != 0) attributes.add("label");
      if ((initBits & INIT_BIT_VALUE) != 0) attributes.add("value");
      return "Cannot build LabelValue, some of required attributes are not set " + attributes;
    }
  }
}
