package io.syndesis.server.metrics.prometheus;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.common.base.MoreObjects;
import com.google.common.collect.Lists;
import com.google.errorprone.annotations.CanIgnoreReturnValue;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import javax.annotation.CheckReturnValue;
import javax.annotation.Generated;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;

/**
 * Immutable implementation of {@link QueryResult}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new QueryResult.Builder()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@Generated({"Immutables.generator", "QueryResult"})
@Immutable
@CheckReturnValue
public final class ImmutableQueryResult implements QueryResult {
  private final String status;
  private final @Nullable String errorType;
  private final @Nullable String error;
  private final @Nullable QueryResult.Data data;

  private ImmutableQueryResult(
      String status,
      @Nullable String errorType,
      @Nullable String error,
      @Nullable QueryResult.Data data) {
    this.status = status;
    this.errorType = errorType;
    this.error = error;
    this.data = data;
  }

  /**
   * @return The value of the {@code status} attribute
   */
  @JsonProperty("status")
  @Override
  public String getStatus() {
    return status;
  }

  /**
   * @return The value of the {@code errorType} attribute
   */
  @JsonProperty("errorType")
  @Override
  public Optional<String> getErrorType() {
    return Optional.ofNullable(errorType);
  }

  /**
   * @return The value of the {@code error} attribute
   */
  @JsonProperty("error")
  @Override
  public Optional<String> getError() {
    return Optional.ofNullable(error);
  }

  /**
   * @return The value of the {@code data} attribute
   */
  @JsonProperty("data")
  @Override
  public Optional<QueryResult.Data> getData() {
    return Optional.ofNullable(data);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link QueryResult#getStatus() status} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for status
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableQueryResult withStatus(String value) {
    if (this.status.equals(value)) return this;
    String newValue = Objects.requireNonNull(value, "status");
    return new ImmutableQueryResult(newValue, this.errorType, this.error, this.data);
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link QueryResult#getErrorType() errorType} attribute.
   * @param value The value for errorType
   * @return A modified copy of {@code this} object
   */
  public final ImmutableQueryResult withErrorType(String value) {
    @Nullable String newValue = Objects.requireNonNull(value, "errorType");
    if (Objects.equals(this.errorType, newValue)) return this;
    return new ImmutableQueryResult(this.status, newValue, this.error, this.data);
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link QueryResult#getErrorType() errorType} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for errorType
   * @return A modified copy of {@code this} object
   */
  public final ImmutableQueryResult withErrorType(Optional<String> optional) {
    @Nullable String value = optional.orElse(null);
    if (Objects.equals(this.errorType, value)) return this;
    return new ImmutableQueryResult(this.status, value, this.error, this.data);
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link QueryResult#getError() error} attribute.
   * @param value The value for error
   * @return A modified copy of {@code this} object
   */
  public final ImmutableQueryResult withError(String value) {
    @Nullable String newValue = Objects.requireNonNull(value, "error");
    if (Objects.equals(this.error, newValue)) return this;
    return new ImmutableQueryResult(this.status, this.errorType, newValue, this.data);
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link QueryResult#getError() error} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for error
   * @return A modified copy of {@code this} object
   */
  public final ImmutableQueryResult withError(Optional<String> optional) {
    @Nullable String value = optional.orElse(null);
    if (Objects.equals(this.error, value)) return this;
    return new ImmutableQueryResult(this.status, this.errorType, value, this.data);
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link QueryResult#getData() data} attribute.
   * @param value The value for data
   * @return A modified copy of {@code this} object
   */
  public final ImmutableQueryResult withData(QueryResult.Data value) {
    @Nullable QueryResult.Data newValue = Objects.requireNonNull(value, "data");
    if (this.data == newValue) return this;
    return new ImmutableQueryResult(this.status, this.errorType, this.error, newValue);
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link QueryResult#getData() data} attribute.
   * A shallow reference equality check is used on unboxed optional value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for data
   * @return A modified copy of {@code this} object
   */
  public final ImmutableQueryResult withData(Optional<? extends QueryResult.Data> optional) {
    @Nullable QueryResult.Data value = optional.orElse(null);
    if (this.data == value) return this;
    return new ImmutableQueryResult(this.status, this.errorType, this.error, value);
  }

  /**
   * This instance is equal to all instances of {@code ImmutableQueryResult} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableQueryResult
        && equalTo((ImmutableQueryResult) another);
  }

  private boolean equalTo(ImmutableQueryResult another) {
    return status.equals(another.status)
        && Objects.equals(errorType, another.errorType)
        && Objects.equals(error, another.error)
        && Objects.equals(data, another.data);
  }

  /**
   * Computes a hash code from attributes: {@code status}, {@code errorType}, {@code error}, {@code data}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + status.hashCode();
    h += (h << 5) + Objects.hashCode(errorType);
    h += (h << 5) + Objects.hashCode(error);
    h += (h << 5) + Objects.hashCode(data);
    return h;
  }

  /**
   * Prints the immutable value {@code QueryResult} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return MoreObjects.toStringHelper("QueryResult")
        .omitNullValues()
        .add("status", status)
        .add("errorType", errorType)
        .add("error", error)
        .add("data", data)
        .toString();
  }

  /**
   * Creates an immutable copy of a {@link QueryResult} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable QueryResult instance
   */
  public static ImmutableQueryResult copyOf(QueryResult instance) {
    if (instance instanceof ImmutableQueryResult) {
      return (ImmutableQueryResult) instance;
    }
    return new QueryResult.Builder()
        .from(instance)
        .build();
  }

  /**
   * Builds instances of type {@link ImmutableQueryResult ImmutableQueryResult}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @NotThreadSafe
  public static class Builder {
    private static final long INIT_BIT_STATUS = 0x1L;
    private long initBits = 0x1L;

    private @Nullable String status;
    private @Nullable String errorType;
    private @Nullable String error;
    private @Nullable QueryResult.Data data;

    /**
     * Creates a builder for {@link ImmutableQueryResult ImmutableQueryResult} instances.
     */
    public Builder() {
      if (!(this instanceof QueryResult.Builder)) {
        throw new UnsupportedOperationException("Use: new QueryResult.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code QueryResult} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final QueryResult.Builder from(QueryResult instance) {
      Objects.requireNonNull(instance, "instance");
      status(instance.getStatus());
      Optional<String> errorTypeOptional = instance.getErrorType();
      if (errorTypeOptional.isPresent()) {
        errorType(errorTypeOptional);
      }
      Optional<String> errorOptional = instance.getError();
      if (errorOptional.isPresent()) {
        error(errorOptional);
      }
      Optional<QueryResult.Data> dataOptional = instance.getData();
      if (dataOptional.isPresent()) {
        data(dataOptional);
      }
      return (QueryResult.Builder) this;
    }

    /**
     * Initializes the value for the {@link QueryResult#getStatus() status} attribute.
     * @param status The value for status 
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("status")
    public final QueryResult.Builder status(String status) {
      this.status = Objects.requireNonNull(status, "status");
      initBits &= ~INIT_BIT_STATUS;
      return (QueryResult.Builder) this;
    }

    /**
     * Initializes the optional value {@link QueryResult#getErrorType() errorType} to errorType.
     * @param errorType The value for errorType
     * @return {@code this} builder for chained invocation
     */
    @CanIgnoreReturnValue 
    public final QueryResult.Builder errorType(String errorType) {
      this.errorType = Objects.requireNonNull(errorType, "errorType");
      return (QueryResult.Builder) this;
    }

    /**
     * Initializes the optional value {@link QueryResult#getErrorType() errorType} to errorType.
     * @param errorType The value for errorType
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("errorType")
    public final QueryResult.Builder errorType(Optional<String> errorType) {
      this.errorType = errorType.orElse(null);
      return (QueryResult.Builder) this;
    }

    /**
     * Initializes the optional value {@link QueryResult#getError() error} to error.
     * @param error The value for error
     * @return {@code this} builder for chained invocation
     */
    @CanIgnoreReturnValue 
    public final QueryResult.Builder error(String error) {
      this.error = Objects.requireNonNull(error, "error");
      return (QueryResult.Builder) this;
    }

    /**
     * Initializes the optional value {@link QueryResult#getError() error} to error.
     * @param error The value for error
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("error")
    public final QueryResult.Builder error(Optional<String> error) {
      this.error = error.orElse(null);
      return (QueryResult.Builder) this;
    }

    /**
     * Initializes the optional value {@link QueryResult#getData() data} to data.
     * @param data The value for data
     * @return {@code this} builder for chained invocation
     */
    @CanIgnoreReturnValue 
    public final QueryResult.Builder data(QueryResult.Data data) {
      this.data = Objects.requireNonNull(data, "data");
      return (QueryResult.Builder) this;
    }

    /**
     * Initializes the optional value {@link QueryResult#getData() data} to data.
     * @param data The value for data
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("data")
    public final QueryResult.Builder data(Optional<? extends QueryResult.Data> data) {
      this.data = data.orElse(null);
      return (QueryResult.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableQueryResult ImmutableQueryResult}.
     * @return An immutable instance of QueryResult
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableQueryResult build() {
      if (initBits != 0) {
        throw new IllegalStateException(formatRequiredAttributesMessage());
      }
      return new ImmutableQueryResult(status, errorType, error, data);
    }

    private String formatRequiredAttributesMessage() {
      List<String> attributes = Lists.newArrayList();
      if ((initBits & INIT_BIT_STATUS) != 0) attributes.add("status");
      return "Cannot build QueryResult, some of required attributes are not set " + attributes;
    }
  }
}
