package io.syndesis.server.metrics.prometheus;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.common.base.MoreObjects;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap;
import com.google.errorprone.annotations.CanIgnoreReturnValue;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.util.Map;
import java.util.Objects;
import javax.annotation.CheckReturnValue;
import javax.annotation.Generated;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;

/**
 * Immutable implementation of {@link QueryResult.Data.Result}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new QueryResult.Data.Result.Builder()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@Generated({"Immutables.generator", "QueryResult.Data.Result"})
@Immutable
@CheckReturnValue
public final class ImmutableResult implements QueryResult.Data.Result {
  private final ImmutableMap<String, String> metric;
  private final ImmutableList<Object> value;

  private ImmutableResult(
      ImmutableMap<String, String> metric,
      ImmutableList<Object> value) {
    this.metric = metric;
    this.value = value;
  }

  /**
   * @return The value of the {@code metric} attribute
   */
  @JsonProperty("metric")
  @Override
  public ImmutableMap<String, String> getMetric() {
    return metric;
  }

  /**
   * @return The value of the {@code value} attribute
   */
  @JsonProperty("value")
  @Override
  public ImmutableList<Object> getValue() {
    return value;
  }

  /**
   * Copy the current immutable object by replacing the {@link QueryResult.Data.Result#getMetric() metric} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the metric map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableResult withMetric(Map<String, ? extends String> entries) {
    if (this.metric == entries) return this;
    ImmutableMap<String, String> newValue = ImmutableMap.copyOf(entries);
    return new ImmutableResult(newValue, this.value);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link QueryResult.Data.Result#getValue() value}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableResult withValue(Object... elements) {
    ImmutableList<Object> newValue = ImmutableList.copyOf(elements);
    return new ImmutableResult(this.metric, newValue);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link QueryResult.Data.Result#getValue() value}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of value elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableResult withValue(Iterable<? extends Object> elements) {
    if (this.value == elements) return this;
    ImmutableList<Object> newValue = ImmutableList.copyOf(elements);
    return new ImmutableResult(this.metric, newValue);
  }

  /**
   * This instance is equal to all instances of {@code ImmutableResult} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableResult
        && equalTo((ImmutableResult) another);
  }

  private boolean equalTo(ImmutableResult another) {
    return metric.equals(another.metric)
        && value.equals(another.value);
  }

  /**
   * Computes a hash code from attributes: {@code metric}, {@code value}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + metric.hashCode();
    h += (h << 5) + value.hashCode();
    return h;
  }

  /**
   * Prints the immutable value {@code Result} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return MoreObjects.toStringHelper("Result")
        .omitNullValues()
        .add("metric", metric)
        .add("value", value)
        .toString();
  }

  /**
   * Creates an immutable copy of a {@link QueryResult.Data.Result} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable Result instance
   */
  public static ImmutableResult copyOf(QueryResult.Data.Result instance) {
    if (instance instanceof ImmutableResult) {
      return (ImmutableResult) instance;
    }
    return new QueryResult.Data.Result.Builder()
        .from(instance)
        .build();
  }

  /**
   * Builds instances of type {@link ImmutableResult ImmutableResult}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @NotThreadSafe
  public static class Builder {
    private ImmutableMap.Builder<String, String> metric = ImmutableMap.builder();
    private ImmutableList.Builder<Object> value = ImmutableList.builder();

    /**
     * Creates a builder for {@link ImmutableResult ImmutableResult} instances.
     */
    public Builder() {
      if (!(this instanceof QueryResult.Data.Result.Builder)) {
        throw new UnsupportedOperationException("Use: new QueryResult.Data.Result.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code Result} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * Collection elements and entries will be added, not replaced.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final QueryResult.Data.Result.Builder from(QueryResult.Data.Result instance) {
      Objects.requireNonNull(instance, "instance");
      putAllMetric(instance.getMetric());
      addAllValue(instance.getValue());
      return (QueryResult.Data.Result.Builder) this;
    }

    /**
     * Put one entry to the {@link QueryResult.Data.Result#getMetric() metric} map.
     * @param key The key in the metric map
     * @param value The associated value in the metric map
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final QueryResult.Data.Result.Builder putMetric(String key, String value) {
      this.metric.put(key, value);
      return (QueryResult.Data.Result.Builder) this;
    }

    /**
     * Put one entry to the {@link QueryResult.Data.Result#getMetric() metric} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final QueryResult.Data.Result.Builder putMetric(Map.Entry<String, ? extends String> entry) {
      this.metric.put(entry);
      return (QueryResult.Data.Result.Builder) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link QueryResult.Data.Result#getMetric() metric} map. Nulls are not permitted
     * @param metric The entries that will be added to the metric map
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("metric")
    public final QueryResult.Data.Result.Builder metric(Map<String, ? extends String> metric) {
      this.metric = ImmutableMap.builder();
      return putAllMetric(metric);
    }

    /**
     * Put all mappings from the specified map as entries to {@link QueryResult.Data.Result#getMetric() metric} map. Nulls are not permitted
     * @param metric The entries that will be added to the metric map
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final QueryResult.Data.Result.Builder putAllMetric(Map<String, ? extends String> metric) {
      this.metric.putAll(metric);
      return (QueryResult.Data.Result.Builder) this;
    }

    /**
     * Adds one element to {@link QueryResult.Data.Result#getValue() value} list.
     * @param element A value element
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final QueryResult.Data.Result.Builder addValue(Object element) {
      this.value.add(element);
      return (QueryResult.Data.Result.Builder) this;
    }

    /**
     * Adds elements to {@link QueryResult.Data.Result#getValue() value} list.
     * @param elements An array of value elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final QueryResult.Data.Result.Builder addValue(Object... elements) {
      this.value.add(elements);
      return (QueryResult.Data.Result.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link QueryResult.Data.Result#getValue() value} list.
     * @param elements An iterable of value elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("value")
    public final QueryResult.Data.Result.Builder value(Iterable<? extends Object> elements) {
      this.value = ImmutableList.builder();
      return addAllValue(elements);
    }

    /**
     * Adds elements to {@link QueryResult.Data.Result#getValue() value} list.
     * @param elements An iterable of value elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final QueryResult.Data.Result.Builder addAllValue(Iterable<? extends Object> elements) {
      this.value.addAll(elements);
      return (QueryResult.Data.Result.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableResult ImmutableResult}.
     * @return An immutable instance of Result
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableResult build() {
      return new ImmutableResult(metric.build(), value.build());
    }
  }
}
